import org.apache.regexp.*;
import java.io.*;
import java.util.*;

/** 
 * Parse a XSL-File and generate Oreilly-like ParameterParser statements.
 * The option "-h" tells the generator to produce code for hidden parameters also.
 * 
 * If using Oreilly's ParameterParser please look at the oreilly_licence.txt file.
 * 
 * @author Klaus Berg
 */
public final class XslParameterParser {

    // The RE Patterns
    private static final RE inputLinePattern = new RE("(<!--.*)|((<input.+(/>|</input>))|(<select[^>]+>)|(<textarea[^>]+>))", RE.MATCH_CASEINDEPENDENT);
    private static final RE hiddenPattern = new RE("type\\s*=\\s*\"hidden\"", RE.MATCH_CASEINDEPENDENT);                                
    private static final RE textPattern = new RE("type\\s*=\\s*\"text\"", RE.MATCH_CASEINDEPENDENT);                                         
    private static final RE cbPattern = new RE("type\\s*=\\s*\"checkbox\"", RE.MATCH_CASEINDEPENDENT);                                       
    private static final RE radioPattern = new RE("type\\s*=\\s*\"radio\"", RE.MATCH_CASEINDEPENDENT);                                       
    private static final RE namePattern = new RE("name\\s*=\\s*\"[^\"]+\"", RE.MATCH_CASEINDEPENDENT);                                       

    private static TreeMap parameterMap = new TreeMap();
    private static TreeMap setterMap = new TreeMap();

    private static boolean generateCodeForHiddenParameters = false;

    public static void main(String[] args) throws Exception {
        if (args.length == 0) {
            System.out.println("usage: java [-h] XslParameterParser <xsl-file1.xsl> <xsl-file2.xsl> ...");
            System.exit(1);
        }
        ArgumentParser parser = new ArgumentParser(args);
        generateCodeForHiddenParameters = parser.hasOption("h");
        String xslPathname = null;
        while ((xslPathname=parser.nextParam()) != null) {
            System.out.println("processing XSL file: " + xslPathname);
            parameterMap.clear();
            setterMap.clear();
            Reader reader = new FileReader(xslPathname);
            // The RE package ReaderCharacterIterator, a "front end"
            // around the Reader object.
            CharacterIterator in = new ReaderCharacterIterator(reader);
            int end = 0;

            File f = new File(xslPathname);
            String xslFilename = f.getName();
            String outputFilename = firstCharToUpperCase(xslFilename.substring(0, xslFilename.length()-4)) + "PP.java";
            System.out.println("generating Java file: " + outputFilename);
            BufferedWriter writer = new BufferedWriter(new FileWriter(outputFilename));
            writer.write("import com.oreilly.servlet.ParameterParser;\n" + "\n");
            writer.write("private ParameterParser parser;" + "\n");
            writer.write("parser = new ParameterParser(request);" + "\n");
            writer.write("\n");
            // For each match in the input, extract and print it.
            while (inputLinePattern.match(in, end)) {
                end = inputLinePattern.getParenEnd(0);
                String line = in.substring(inputLinePattern.getParenStart(0), end);
                if (line.startsWith("<!--")) {
                    continue; // ignore comments
                }
                if (line.startsWith("<input")) {
                    processInputTextLine(line);
                }
                else if (line.startsWith("<select")) {
                    processSelectLine(line);
                }
                else if (line.startsWith("<textarea")) {
                    processTextareaLine(line);
                }
            }

            try {
                Set set = parameterMap.entrySet();
                Iterator iter = set.iterator();
                while (iter.hasNext()) {
                    Map.Entry me = (Map.Entry)iter.next();
                    writer.write((String)me.getValue() + "\n");
                }
                writer.write("\n");
                set = setterMap.entrySet();
                iter = set.iterator();
                while (iter.hasNext()) {
                    Map.Entry me = (Map.Entry)iter.next();
                    writer.write((String)me.getValue() + "\n");
                }
                writer.close();
            }
            catch (IOException e) {
                e.printStackTrace();
                System.exit(1);
            }
        }
    }

    private static void processInputTextLine(String line) {
        if (generateCodeForHiddenParameters && hiddenPattern.match(line)) {
            if (namePattern.match(line)) {
                processStringParameter(line, "input-hidden parameter ");
            }
        }
        else if (textPattern.match(line)) {
            if (namePattern.match(line)) {
                processStringParameter(line, "input-text parameter ");
            }
        }
        else if (cbPattern.match(line)) {
            if (namePattern.match(line)) {
                processBooleanParameter(line, "input-checkbox parameter ");
            }
        }
        else if (radioPattern.match(line)) {
            if (namePattern.match(line)) {
                processStringParameter(line, "input-radio parameter ");
            }
        }
    }

    private static void processSelectLine(String line) {
        if (namePattern.match(line)) {
            processStringParameter(line, "select parameter ");
        }
    }

    private static void processTextareaLine(String line) {
        if (namePattern.match(line)) {
            processStringParameter(line, "textarea parameter ");
        }
    }

    private static void processStringParameter(String line, String msg) {
        String nameLine = line.substring(namePattern.getParenStart(0), namePattern.getParenEnd(0));
        String parameterName = nameLine.substring(nameLine.indexOf('"'));
        String p = parameterName.substring(1, parameterName.length()-1);
        String javaParameterName = createJavaParameterName(p);
        String methodVariable = "";        
        if (msg.startsWith("input-hidden") && generateCodeForHiddenParameters && parameterName.indexOf('{') == -1) {
            methodVariable = "String " + javaParameterName + " = parser.getStringParameter(" + parameterName + ", \"\"); // " + msg +parameterName;
            parameterMap.put(javaParameterName, methodVariable);
        }
        if (parameterName.indexOf('{') == -1) {
            methodVariable = "String " + javaParameterName + " = parser.getStringParameter(" + parameterName + ", \"\"); // " + msg +parameterName;
        }
        else {
            String arrayParameterName = createArrayParameterName(p);
            String msg1 = msg + "ARRAY-ELEMENT ";
            methodVariable = "String " + javaParameterName + " = parser.getStringParameter(" + arrayParameterName + ", \"\"); // " + msg1 +parameterName;
        }
        parameterMap.put(javaParameterName, methodVariable);
        //if (generateCodeForHiddenParameters && msg.startsWith("input-hidden")) {
        String callingSetter = "model.set" + firstCharToUpperCase(javaParameterName) + "(" + javaParameterName + ");";
        setterMap.put(javaParameterName, callingSetter);
        //}
    }


    private static void processBooleanParameter(String line, String msg) {
        String nameLine = line.substring(namePattern.getParenStart(0), namePattern.getParenEnd(0));
        String parameterName = nameLine.substring(nameLine.indexOf('"'));
        String p = parameterName.substring(1, parameterName.length()-1);
        String javaParameterName = createJavaParameterName(p);
        String methodVariable;
        if (parameterName.indexOf('{') == -1) {
            methodVariable = "boolean " + javaParameterName + " = parser.getBooleanParameter(" + parameterName + ", false); // " + msg +parameterName;
        }
        else {
            String arrayParameterName = createArrayParameterName(p);
            String msg1 = msg + "ARRAY-ELEMENT ";
            methodVariable = "boolean " + javaParameterName + " = parser.getBooleanParameter(" + arrayParameterName + ", false); // " + msg1 +parameterName;
        }
        parameterMap.put(javaParameterName, methodVariable);
        String callingSetter = "model.set" + firstCharToUpperCase(javaParameterName) + "(" + javaParameterName + ");";
        setterMap.put(javaParameterName, callingSetter);
    }

    private static String createJavaParameterName(String s) {
        String parameterName = s.toLowerCase();
        if (parameterName.indexOf('-') != -1) {
            parameterName = parameterName.replace('-', '_');
        }
        int i;
        if ((i=parameterName.indexOf('{')) != -1) {
            StringBuffer sbuf = new StringBuffer(s.length()+10);
            parameterName = parameterName.substring(0, i-1);
        }
        return parameterName;
    }


    private static String createArrayParameterName(String s) {
        String parameterName = s.toLowerCase();        
        if (parameterName.indexOf('-') != -1) {
            parameterName = parameterName.replace('-', '_');
        }
        if (parameterName.indexOf('/') != -1) {
            parameterName = parameterName.replace('/', '_');
        }
        if (parameterName.indexOf('@') != -1) {
            parameterName = parameterName.replace('@', '_'); 
        }
        if (parameterName.indexOf('}') != -1) {
            parameterName = parameterName.replace('}', '_');
        }
        int i;
        if ((i=parameterName.indexOf('{')) != -1) {
            StringBuffer sbuf = new StringBuffer(s.length()+10);
            parameterName = sbuf.append("\"").append(parameterName.substring(0, i-1)).append("\" + ").append(parameterName.substring(i+1)).toString();
        }
        return parameterName;
    }

    private static String firstCharToUpperCase(String s) {
        return Character.toUpperCase(s.charAt(0)) + s.substring(1);
    }
}

